"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _MicroRedisClient_instances, _MicroRedisClient_sendPacket;
Object.defineProperty(exports, "__esModule", { value: true });
exports.MicroRedisClient = void 0;
const micro_1 = require("@ipare/micro");
const connection_1 = require("./connection");
class MicroRedisClient extends micro_1.MicroClient {
    constructor(options = {}) {
        super();
        this.options = options;
        _MicroRedisClient_instances.add(this);
        connection_1.initRedisConnection.bind(this)();
    }
    connect() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.initClients();
        });
    }
    /**
     * for @ipare/inject
     */
    dispose() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.closeClients();
        });
    }
    send(pattern, data, timeout) {
        const _super = Object.create(null, {
            createPacket: { get: () => super.createPacket }
        });
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            if (!((_a = this.sub) === null || _a === void 0 ? void 0 : _a.isReady) || !((_b = this.pub) === null || _b === void 0 ? void 0 : _b.isReady)) {
                return {
                    error: "The connection is not connected",
                };
            }
            pattern = this.prefix + pattern;
            const packet = _super.createPacket.call(this, pattern, data, true);
            const reply = pattern + "." + packet.id;
            const sub = this.sub;
            return new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
                var _c;
                let timeoutInstance;
                const sendTimeout = (_c = timeout !== null && timeout !== void 0 ? timeout : this.options.sendTimeout) !== null && _c !== void 0 ? _c : 10000;
                if (sendTimeout != 0) {
                    timeoutInstance = setTimeout(() => {
                        sub.unsubscribe(reply);
                        resolve({
                            error: "Send timeout",
                        });
                    }, sendTimeout);
                }
                yield sub.subscribe(reply, (buffer) => {
                    if (timeoutInstance) {
                        clearTimeout(timeoutInstance);
                        timeoutInstance = undefined;
                    }
                    sub.unsubscribe(reply);
                    (0, micro_1.parseBuffer)(buffer, (packet) => {
                        var _a;
                        resolve({
                            data: (_a = packet.data) !== null && _a !== void 0 ? _a : packet.response,
                            error: packet.error,
                        });
                    });
                }, true);
                __classPrivateFieldGet(this, _MicroRedisClient_instances, "m", _MicroRedisClient_sendPacket).call(this, packet);
            }));
        });
    }
    emit(pattern, data) {
        var _a, _b;
        if (!((_a = this.sub) === null || _a === void 0 ? void 0 : _a.isReady) || !((_b = this.pub) === null || _b === void 0 ? void 0 : _b.isReady)) {
            throw new Error("The connection is not connected");
        }
        pattern = this.prefix + pattern;
        const packet = super.createPacket(pattern, data, false);
        __classPrivateFieldGet(this, _MicroRedisClient_instances, "m", _MicroRedisClient_sendPacket).call(this, packet);
    }
}
exports.MicroRedisClient = MicroRedisClient;
_MicroRedisClient_instances = new WeakSet(), _MicroRedisClient_sendPacket = function _MicroRedisClient_sendPacket(packet) {
    const json = JSON.stringify(packet);
    const str = `${json.length}#${json}`;
    const pub = this.pub;
    pub.publish(packet.pattern, str);
};
